<?php
/**
 * $Horde: imp/redirect.php,v 1.110 2004/12/04 23:25:25 chuck Exp $
 *
 * Copyright 1999-2004 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 1999-2004 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('AUTH_HANDLER', true);
@define('IMP_BASE', dirname(__FILE__));
$authentication = 'none';
require_once IMP_BASE . '/lib/base.php';
require_once IMP_BASE . '/lib/Session.php';

/* Get the actionID. */
$actionID = (Util::getFormData('action') == 'compose') ? 'login_compose' : Util::getFormData('actionID');

$autologin = Util::getFormData('autologin');
$imapuser = empty($autologin) ? Util::getPost('imapuser') : Auth::getBareAuth();
$pass = empty($autologin) ? Util::getPost('pass') : Auth::getCredential('password');

/* If we already have a session: */
if (isset($_SESSION['imp']) && is_array($_SESSION['imp'])) {
    /* Make sure that if a username was specified, it is the current
     * username. */
    if ((is_null($imapuser) || ($imapuser == $_SESSION['imp']['user'])) &&
        (is_null($pass) || ($pass == Secret::read(Secret::getKey('imp'), $_SESSION['imp']['pass'])))) {

        $isLogin = IMP_Session::loginFlag();

        IMP_Session::loginTasks();

        $url = Util::getFormData('url');
        if (empty($url)) {
            $url = IMP::getInitialUrl($actionID, false);
        } elseif (!empty($actionID)) {
            $url = Util::addParameter($url, 'actionID', $actionID);
        }

        /* Don't show popup window in initial page. */
        if ($isLogin) {
            $url = Util::addParameter($url, 'no_newmail_popup', 1);
        }

        if (Util::getFormData('load_frameset')) {
            $full_url = Horde::applicationUrl($registry->get('webroot', 'horde') . '/index.php', true);
            $url = Util::addParameter($full_url, 'url', $url, false);
        }

        header('Refresh: 0; URL=' . $url);
        exit;
    } else {
        /* Disable the old session. */
        unset($_SESSION['imp']);
        $url = Auth::addLogoutParameters(IMP::logoutUrl(), AUTH_REASON_FAILED);
        header('Location: ' . $url);
        exit;
    }
}

/* Create a new session if we're given the proper parameters. */
if ((!is_null($imapuser) && !is_null($pass))) {
    if (Auth::getProvider() == 'imp') {
        /* Destroy any existing session on login and make sure to use
         * a new session ID, to avoid session fixation issues. */
        Horde::getCleanSession();
    }

    /* If we're not using hordeauth get the required parameters from
     * the form data. */
    if (empty($autologin)) {
        $sessArray = array(
            'server' => Util::getFormData('server'),
            'realm' => Util::getFormData('realm'),
            'port' => Util::getFormData('port'),
            'protocol' => Util::getFormData('protocol'),
            'maildomain' => Util::getFormData('maildomain'),
            'namespace' => Util::getFormData('namespace'),
            'folders' => Util::getFormData('folders')
        );
    } else {
        /* Read the required server parameters from the servers.php
         * file. */
        require_once IMP_BASE . '/config/servers.php';
        $server_key = Util::getFormData('server_key');
        if (!empty($servers[$server_key])) {
            $sessArray = $servers[$server_key];
            if (!empty($sessArray['hordeauth']) &&
                strcasecmp($sessArray['hordeauth'], 'full') == 0) {
                $imapuser = Auth::getAuth();
            }
        } else {
            $entry = sprintf('Invalid server key from client [%s]', $_SERVER['REMOTE_ADDR']);
            Horde::logMessage($entry, __FILE__, __LINE__, PEAR_LOG_INFO);
        }
    }

    if (IMP_Session::createSession($imapuser, $pass, $sessArray['server'], $sessArray)) {
        $entry = sprintf('Login success for %s [%s] to {%s:%s}', $imp['uniquser'], $_SERVER['REMOTE_ADDR'], $imp['server'], $imp['port']);
        Horde::logMessage($entry, __FILE__, __LINE__, PEAR_LOG_NOTICE);

        $ie_version = Util::getFormData('ie_version');
        if ($ie_version) {
            $browser->setIEVersion($ie_version);
        }

        if (($horde_language = Util::getFormData('new_lang'))) {
            $_SESSION['horde_language'] = $horde_language;
        }

        $isLogin = IMP_Session::loginFlag();

        IMP_Session::loginTasks();

        $url = '';
        if (Util::getFormData('url')) {
            $url = Horde::url(Util::removeParameter(Util::getFormData('url'), session_name()));
        } elseif (Auth::getProvider() == 'imp') {
            $url = Horde::applicationUrl($registry->get('webroot', 'horde') . '/', true);
            /* Force the initial page to IMP if we're logging in to
             * compose a message. */
            if ($actionID == 'login_compose') {
                $url = Util::addParameter($url, 'url', IMP::getInitialUrl('login_compose', false));
            }
        } else {
            $url = IMP::getInitialUrl();
            /* Don't show popup window in initial page. */
            if ($isLogin) {
                $url = Util::addParameter($url, 'no_newmail_popup', 1);
            }
        }

        /* Preserve the actionID. */
        if ($actionID) {
            $url = Util::addParameter($url, 'actionID', $actionID, false);
        }
    } else {
        $url = Auth::addLogoutParameters(IMP::logoutUrl());
    }

    if (Util::getFormData('load_frameset')) {
        $full_url = Horde::applicationUrl($registry->get('webroot', 'horde') . '/', true);
        $url = Util::addParameter($full_url, 'url', $url, false);
    }

    header('Refresh: 0; URL=' . $url);
    exit;
}

/* No session, and no login attempt. Just go to the login page. */
require IMP_BASE . '/login.php';
